import threading
import socket
import sys
import logging
import npyscreen
from LoginForm import LoginForm
from ChatUI import ChatUI
from Emitter import Emitter

class Client:
    def __init__(self, host, port):
        self.host = host
        self.port = port
        self.port = int(self.port)
        logging.basicConfig(filename='logs.log', level = logging.DEBUG, format = '[%(asctime)s] %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
        logging.debug("Begin new logging here")
        #if self.port != 5344:
        #    print("stop, unauthorized")
        #    quit()
        self.enabled = True
        self.sessionid = None
        self.inChannel = False
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.channelSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        print("connecting...")
        self.socket.connect((self.host, self.port))
        self.asyncSocketThread = threading.Thread(target = self.__socketThread, args = (self.socket, self.sessionid), daemon = True)
        self.scope = "[global]"

        try:
            self.app = ClientApp()
            emitter = Emitter.emitter
            # events
            @emitter.on('logged')
            def loginHandler():
                self.asyncSocketThread.start()

            @emitter.on('newchannel')
            def channelHandler(stuff):
                port = stuff[1]
                self.inChannel = True
                print("Connecting to {}".format(stuff[0]))
                self.scope = stuff[0]
                self.channelSocket.connect((self.host, port))
                new = threading.Thread(target = self.__channelSocketThread, args = (self.channelSocket, self.sessionid), daemon = True)
                new.start()

            @emitter.on('send')
            def chatMessageHandler(message):
                if message[0] == "" or not message[0]:
                    return False
                finalMessage = message[1] + "|" + message[0] + "|" + message[2]
                if self.inChannel:
                    self.channelSocket.sendall(finalMessage.encode('utf-8'))
                else:
                    self.socket.sendall(finalMessage.encode('utf-8'))

            self.app.run() # loop stops here, so we'll wait its end

        except KeyboardInterrupt:
            self.app.switchForm(None)
            print("Closed UI, ^C again to exit")

    def __channelSocketThread(self, socket, session):
        while self.inChannel:
            data = socket.recv(1024)
            if not data:
                Emitter.emitter.emit('bye', 'Connection closed')
                self.inChannel = False
                break
            logging.debug("Ricevuto {}".format(data.decode('utf-8')))
            message = data.decode('utf-8')
            if message[0] == "!":
                # it's a command from the server
                if message.split(":")[0] == "!session":
                    # server is verifying my session, I better have it!
                    Emitter.emitter.emit('session')
                if message.split(":")[0] == "!newchannel":
                    print("Connecting to a new channel...")
                    #self.socket.close()
                    Emitter.emitter.emit('newchannel', (message.split(":")[1], int(message.split(":")[2])))
                    continue

            Emitter.emitter.emit('chatMessage', (message))

    def __socketThread(self, socket, sessionid):
        logging.debug("ready!")
        while self.enabled:
            data = socket.recv(1024)
            if not data:
                logging.debug("closing this")
                Emitter.emitter.emit('bye', 'Connection closed')
                self.enabled = False
                break
            logging.debug("Ricevuto {}".format(data.decode('utf-8')))
            message = data.decode('utf-8')
            if message[0] == "!":
                # it's a command from the server
                if message.split(":")[0] == "!session":
                    # server is verifying my session, I better have it!
                    Emitter.emitter.emit('session')
                if message.split(":")[0] == "!newchannel":
                    print("Connecting to a new channel...")
                    #self.socket.close()
                    Emitter.emitter.emit('newchannel', (message.split(":")[1], int(message.split(":")[2])))
                    continue

            Emitter.emitter.emit('chatMessage', (message))


class ClientApp(npyscreen.NPSAppManaged):
    def onStart(self):
        self.registerForm("MAIN", LoginForm())
        self.registerForm("CHAT", ChatUI())
        self.username = None
        self.sessionid = None

        # self.emitter.emit('evento', roba)
        # @emitter.on('evento')
        # def handler(roba): pass

    def getEmitter(self):
        return self.emitter

if __name__ == '__main__':
    client = Client('172.20.16.14', 7777)

'''class Client:
    def __init__(self, host, port):
        self.port = port
        self.host = host
        self.logged = False
        self.socket = None
        self.enabled = True
        self.scope = "[global]"
        self.username = None
        self.listen = threading.Thread(target = self.__start, args = (), daemon = True)
        self.input = threading.Thread(target = self.__input, args = (), daemon = True)

        # client estabilishes connection and starts login routine
        self.listen.start()

    def disable(self):
        self.enabled = False

    def __start(self):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            self.socket = s
            print("Connessione a {}".format(self.host))
            self.socket.connect((self.host, self.port))
            self.input.start()
            while self.enabled == True:
                data = self.socket.recv(1024)
                message = data.decode('utf-8')

                if message != "":
                    if message[0] == "!":
                        cmd = message.lower()
                        # commands received from the server
                        args = cmd.split(":")
                        if args[0] == "!newchannel":
                            # new channel incoming, do the thing
                            continue

                        if args[0] == "!logincorrect":
                            # login is correct, setting new username
                            # will set session in the future

                            self.logged = True
                            self.username = args[1]
                            self.scope = args[2]

                        if args[0] == "!newscope":
                            print("new scope")
                            # setting a new scope from the server, could be login, global or everything concerning channels
                            self.scope = args[1] # set the scope and prefix in chat
                    else: # not a command
                        if data == "" or not data:
                            continue

                        npyscreen.notify_confirm(message)

    def __input(self):
        while self.enabled:
            if self.scope == "[login]":
                if self.logged == False:
                    username, password = npyscreen.wrapper_basic(initLogin)
                    sendable = "!login:"+username + ":" + password
                    self.socket.sendall(sendable.encode('utf-8'))
            else:
                print("ASDSD")
                if self.logged:
                    chat = npyscreen.wrapper_basic(initChatUI)
                msg = input(self.scope + " ")
                if not msg or msg == "" or msg == "\n":
                    continue

                if msg[0] == "!":
                    cmd = msg.lower()
                    pieces = msg.split(":")

                    if cmd == "!channellist":
                        self.socket.sendall(cmd.encode('utf-8'))

                    if pieces[0] == "!join":
                        channel = pieces[1]
                        sendable = "!join:"+channel
                        self.socket.sendall(cmd.encode('utf-8'))

            msg = input(self.scope + " ")

            if not msg or msg == "" or msg == "\n":
                continue

            if msg[0] == "!":
                cmd = msg.lower()
                pieces = msg.split(":")

                if cmd == "!channellist":
                    self.socket.sendall(cmd.encode('utf-8'))

                if pieces[0] == "!join":
                    channel = pieces[1]
                    sendable = "!join:"+channel
                    self.socket.sendall(cmd.encode('utf-8'))

            if self.scope == "[login]":
                print("login scope")
                username, password = npyscreen.wrapper_basic(initLogin)
                sendable = "!login:"+username + ":" + password
                self.socket.sendall(sendable.encode('utf-8'))
            else:
                sendable = self.username + "|" + msg
                self.socket.sendall(sendable.encode('utf-8'))'''

'''
    def __waitInput(self):
        while self.waiting == True:
            msg = input(self.scope + " ")

            if not msg or msg == "" or msg == "\n":
                continue

            if msg[0] == "!":
                cmd = msg.lower();
                print(msg)
                pieces = msg.split(":")
                if pieces[0] == "!join":
                    channel = pieces[1]
                    sendable = "!join:"+channel
                    self.socket.sendall(cmd.encode('utf-8'))

                if cmd == "!channellist":
                    self.socket.sendall(cmd.encode('utf-8'))

            if self.scope == "[login]":
                username, password = npyscreen.wrapper_basic(initLogin)
                sendable = "!login:"+username + ":" + password
                self.socket.sendall(sendable.encode('utf-8'))
            else:
                sendable = self.username + "|" + msg
                self.socket.sendall(sendable.encode('utf-8'))


    def __waitInput(self):
        while self.waitingForInput == True:

            msg = input(self.scope)
            if not msg:
                break
            print("MESSAGE SENT: " + msg)
            if(self.scope == "[login]"):
                if msg.count(":") == 1:
                    msg = msg.split(":")[0] + ":" + msg.split(":")[1]
                    self.socket.sendall(msg.encode(encoding = 'UTF-8'))
                else:
                    self.scope = "[login]"
                    print("Formato errato")
                    break
            else:
                if msg[0] == "!":
                    pass
            msg = self.username + "|" + msg
            self.socket.sendall(msg.encode(encoding = 'UTF-8')'''
